#!/usr/bin/fades

import argparse
import io
import os
import pickle
import smtplib
import sys

from email.mime.text import MIMEText

import infoauth  # fades
import twitter  # fades

USERINFO = """\
    {name} [{location}]  https://twitter.com/{screenname}
        '{description}'
        Followed by {followers}, follows to {friends} (following? {following})

"""


class Twitter:
    def __init__(self):
        auth = infoauth.load("/home/facundo/.twapp.auth")
        self._api = twitter.Twitter(auth=twitter.NoAuth())
        # self._api = twitter.Twitter(auth=twitter.OAuth(
        #     auth['access_token'], auth['access_secret'],
        #     auth['consumer_token'], auth['consumer_secret']))
        self._username = "facundobatista"

    def get_current_followers(self):
        """Get the set of ids of current followers."""
        cursor = -1
        all_followers = set()
        while cursor:
            resp = self._api.followers.ids(
                screen_name=self._username, cursor=cursor)
            all_followers.update(resp["ids"])
            cursor = resp['next_cursor']
        return all_followers

    def get_user_info(self, user_ids):
        """Return some info for the list of users."""
        if not user_ids:
            return {}
        resp = self._api.users.lookup(user_id=",".join(map(str, user_ids)))
        result = {}
        for item in resp:
            data = dict(
                name=item['name'],
                location=item['location'],
                followers=item['followers_count'],
                description=item['description'],
                following=item['following'],
                friends=item['friends_count'],
                screenname=item['screen_name'],
            )
            result[item['id']] = data
        return result


def _send_mail(text, short):
    msg = MIMEText(text)

    msg['Subject'] = 'Reporte Pelícano: ' + short
    msg['From'] = "bot@taniquetil.com.ar"
    msg['To'] = "facundo@taniquetil.com.ar"

    # Send the message via our own SMTP server.
    s = smtplib.SMTP('smtp.gmail.com:587')
    s.starttls()
    auth = infoauth.load("/home/facundo/.mailbot.auth")
    s.login(auth['user'], auth['password'])
    s.send_message(msg)
    s.quit()


def main(state_file, tostdout):
    """Main entry point."""
    tw = Twitter()

    if tostdout:
        output = sys.stdout
    else:
        output = io.StringIO()

    if os.path.exists(state_file):
        with open(state_file, 'rb') as fh:
            prev_followers = pickle.load(fh)
    else:
        prev_followers = {}

    # conversion from old format
    if isinstance(prev_followers, set):
        prev_followers = dict.fromkeys(prev_followers)

    # gradual improvement of stored dict (as per conversion above): get 50 users
    # still-with-no-data and fill them
    incomplete = [user_id for user_id, info in prev_followers.items() if info is None]
    print("WARNING: {} incomplete items in the state".format(len(incomplete)), file=output)
    user_info = tw.get_user_info(incomplete[:50])
    prev_followers.update(user_info)

    # get followers difference
    current_followers_set = tw.get_current_followers()
    fw_gained = current_followers_set - set(prev_followers)
    fw_lost = set(prev_followers) - current_followers_set

    # have a dict with followers and all the info
    user_info = tw.get_user_info(fw_gained)
    all_anytime_followers = prev_followers.copy()
    all_anytime_followers.update(user_info)

    # save the current_followers with their info
    current_followers = {uid: all_anytime_followers[uid] for uid in current_followers_set
                         if uid in all_anytime_followers}
    with open(state_file, 'wb') as fh:
        pickle.dump(current_followers, fh)

    if not prev_followers:
        print("Special run: just store current followers:", len(current_followers), file=output)
        return

    print("New followers ({})".format(len(fw_gained)), file=output)
    for follower in fw_gained:
        info = all_anytime_followers.get(follower)
        if info is not None:
            print(USERINFO.format(**info), file=output)
        else:
            print("WARNING: Also new follower {}, no data".format(follower), file=output)

    print("Lost followers ({})".format(len(fw_lost)), file=output)
    for follower in fw_lost:
        info = all_anytime_followers.get(follower)
        if info is not None:
            print(USERINFO.format(**info), file=output)
        else:
            print("WARNING: Also lost follower {}, no data".format(follower), file=output)

    if not tostdout:
        abstract = "[{}/{}]".format(len(fw_gained), len(fw_lost))
        val = output.getvalue()
        try:
            _send_mail(val, abstract)
        except Exception:
            # on any error, dump the info to stderr
            print("Abstract:", repr(abstract), file=sys.stderr)
            print("Content:\n", file=sys.stderr)
            print(val, file=sys.stderr)
            raise


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Tells who started/stopped following you in Twitter.")
    parser.add_argument('--stdout', default=False, action='store_true')
    parser.add_argument('state_file')

    args = parser.parse_args()
    main(args.state_file, args.stdout)
